package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils._

/** A simple test (x1,x2,x3,...)_l -> y_l distribution dependence.
  * @version $Id: ExampleDistribution1Stage.scala,v 1.3 2015/11/28 08:07:19 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  * A library from http://www.ioffe.ru/LNEPS/malyshkin/code_polynomials_quadratures.zip is required.
  */
class ExampleDistribution1Stage(val dimX:Int,val Q:OrthogonalPolynomialsABasis) {

  val RANDOM=new java.util.Random(123)
  val Runge=new PolynomialsRatio(Array(1,0,25),Array(1),new OrthogonalPolynomialsMonomialsBasis())
  val FUNCS=Array((x:Double)=>x,(x:Double)=>Runge.getF(x),(x:Double)=>if(x<=0) 0.0 else 1.0)

  def getSample(N:Int,M:Int,R:Double,f:(Double)=>Double)={
    val r=new Array[OneYObservationPoint](M)
    for(l <- 0 until M)
    {
      val x=2*RANDOM.nextDouble()-1
      val xa=new Array[Double](N)
      val y=f(x)
      for(j <- 0 until N)
      {
        xa(j)=x+R*(2*RANDOM.nextDouble()-1)
      }
      r(l)=new OneYObservationPoint(y,xa)
    }
    r
  }

def printOneStepYDistribution(txt:String,N:Int,M:Int,R:Double,f: (Double) => Double) = {
  val sample=getSample(N,M,R,f)
  val yd=new DistributionRegression1Step(dimX,sample,Q)
  val fname="OUT.R="+R+".d="+yd.dim+"."+txt+".dat";
  val xmin= -1.5
  val xmax= 1.5
  val step=(xmax-xmin)/10000
  System.err.print("Writing "+fname);
  val of=new java.io.PrintWriter(fname)
  of.print("|x\ty\tLS\tRN")
  for(k <- 0 until yd.dim)  of.print("\tP("+yd.ev(k)+")")
  of.println()
  for(x <- xmin to xmax+step/2 by step)
  {
    val y=f(x)
    of.print(""+x+"\t"+y+"\t"+yd.getYLSQ(x)+"\t"+yd.getYRN(x))
    for(k <- 0 until yd.dim) {
      val(yn,pP)=yd.getYkP(x)
      for(k <- 0 until yd.dim) of.print("\t"+pP(k))
    }
    of.println()
  }
  of.close()
  System.err.println(" DONE");
}
}

object ExampleDistribution1Stage {

  def main(args: Array[String]) {
    val N=1000
    val M=10000
    for(d <- List(20,10)) {
      val E=new ExampleDistribution1Stage(d,new OrthogonalPolynomialsChebyshevBasis()/* new OrthogonalPolynomialsLegendreBasis()*/)
      for(R <- List(0.01,0.1,0.3,0.5))
      {
        E.printOneStepYDistribution("X",N,M,R,E.FUNCS(0))
        E.printOneStepYDistribution("Runge",N,M,R,E.FUNCS(1))
        E.printOneStepYDistribution("Step",N,M,R,E.FUNCS(2))
      }
    }
  }
}
