package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils._

/** A simple image interpolation. Uses Legendre or Chebyshev moments.
  * @version $Id: ExampleImageInterpolation.scala,v 1.33 2015/11/17 15:52:23 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  * A library from http://www.ioffe.ru/LNEPS/malyshkin/code_polynomials_quadratures.zip is required.
  */
class ExampleImageInterpolation(val fname:String,val Q:OrthogonalPolynomialsABasis) {
  val image=javax.imageio.ImageIO.read(new java.io.File(fname));
  val origraster=
  {
    System.err.println("Read from "+fname+" w="+image.getWidth()+" h="+image.getHeight()+" imagetype="+image.getType()+" basis="+Q)
    image.getData().getPixels(0, 0, image.getWidth(), image.getHeight(), new Array[Double](image.getWidth()*image.getHeight()))
  }
  def getImageFromRaster(w:Int,h:Int,r:Array[Double]):java.awt.image.BufferedImage=
  {
    if(r.length!=(w*h)) throw new IllegalArgumentException("r.length="+r.length+" w="+w+" h="+h)
    val rimg = new java.awt.image.BufferedImage(w, h, java.awt.image.BufferedImage.TYPE_BYTE_GRAY /*TYPE_USHORT_GRAY*/)
    val wr=rimg.getRaster()
    val px=new Array[Double](1)
    for (y  <-  0 until h) {
      for (x <- 0 until w) {
        //System.err.println("pixel("+x+","+y+")="+r(x+y*w))
        px(0)=r(x+y*w)
        wr.setPixel(x, y, px)
      }
    }
    rimg
  }
  def saveImage(suffix:String,img:java.awt.image.BufferedImage)={
    val fsave=fname+suffix+".png"
    System.err.println("Saving to "+fsave)
    javax.imageio.ImageIO.write(img, "png", new java.io.File(fsave));
  }

  /** Calculate moments from raster.
    * Thye precision is about 1/w or 1/h.
    */
  def getMomentFromRaster(nx:Int,ny:Int,w:Int,h:Int,r:Array[Double]):Array[Double]={
    System.err.println("Moments calculate enter");
    if(r.length!=(w*h)) throw new IllegalArgumentException("r.length="+r.length+" w="+w+" h="+h)
    val moms=new Array[Double](nx*ny);
    // Very crude way to estimate moments.
    // I know about possible computational optimization and that the basis values
    // have to be calculated recurrently instead of calling Q.B.calculate
    val scalex:Double=w-1
    val scaley:Double=h-1
    val bX=Array.ofDim[Double](nx,w)
    val bY=Array.ofDim[Double](ny,h)
    for (ox <- 0 until w ; mx <- 0 until nx){
      val x=ox/scalex // [0..1]
      bX(mx)(ox)=Q.B.calculate(mx,2*x-1)
    }
    for (oy <- 0 until h ; my <- 0 until ny){
      val y=oy/scaley // [0..1]
      bY(my)(oy)=Q.B.calculate(my,2*y-1)
    }

    for( my <- 0 until ny ;  mx <- 0 until nx) {
      val bx=bX(mx)
      val by=bY(my)
      var s=0.0;
      for (oy <- 0 until h ; ox <- 0 until w){
        s+=r(ox+oy*w)*bx(ox)*by(oy)
      }
      moms(mx+nx*my)=s
    }

    /*
    // Check basis precision
    val aY=new Array[Double](ny)
    for (oy  <-  0 until h) {
      val y=oy/scaley // [0..1]
      for( my <- 0 until ny) {
        aY(my)+=Q.B.calculate(my,2*y-1)*w
      }
    }
    System.err.println("aY="+java.util.Arrays.toString(aY)+" should be=0 with about 1/h h="+h+" precision, B="+Q.B)
     */  

    System.err.println("Moments calculate exit");

    moms
  }

  def getMomensFromConsant_Sample(nx:Int,ny:Int,w:Int,h:Int):Array[Double]={
    val r=new Array[Double](w*h);
    java.util.Arrays.fill(r,1);
    getMomentFromRaster(nx,ny,w,h,r)
  }


  def calculateQQMatrix(dimx:Int,dimy:Int,nx:Int,ny:Int,moms:Array[Double]):Array[Double]={
    if(2*dimx-1>nx) throw new IllegalArgumentException("dimx="+dimx+" nx="+nx)
    if(2*dimy-1>ny) throw new IllegalArgumentException("dimy="+dimy+" ny="+ny)
    val cX=new Array[Double](2*dimx-1)
    val cY=new Array[Double](2*dimy-1)
    val n=dimx*dimy
    val QQ=new Array[Double](n*n)    

    System.err.println("QQ calculate enter");
    for(ly <- 0 until dimy ; my <- 0 until dimy) {
      Q.setQQCoefs(ly,my,cY)
      for(lx <- 0 until dimx ; mx <- 0 until dimx) {
        Q.setQQCoefs(lx,mx,cX)
        QQ(lx+ly*dimx+n*(mx+my*dimx))={
          var s=0.0
          for(jy <- 0 to ly+my ; jx <- 0  to lx+mx) s+=cY(jy)*cX(jx)*moms(jx+nx*jy)
          s
        }
      }
    }
    System.err.println("QQ calculate exit");
    QQ
  }

  def printConstantMoments(nx:Int,ny:Int,w:Int,h:Int)={
    val mom_sample=getMomensFromConsant_Sample(nx,ny,w,h)
     for( my <- 0 until ny ; mx <- 0 until nx) {
         System.err.println("mx="+mx+" my="+my+" sample="+mom_sample(mx+nx*my))
     }
  }

  def printQQMatrx(dimx:Int,dimy:Int,QQList:List[Array[Double]])
  {
    val n=dimx*dimy
    for(ly <- 0 until dimy ; lx <- 0 until dimx) {
      for(my <- 0 until dimy ; mx <- 0 until dimx) {
        System.err.print("matrs:[ly="+ly+" lx="+lx+"; my="+my+" mx="+mx+"]=")
        for(QQ <- QQList)
        {
          //System.err.println("QQ.length="+QQ.length+" dimx="+dimx+" dimy="+dimy)          
          System.err.print(";"+QQ(lx+dimx*ly+n*(mx+dimx*my)))
        }
        System.err.println()
      }
    }
  }

  def printQQConstant(dimx:Int,dimy:Int,w:Int,h:Int)
  {
    val nx=2*dimx-1
    val ny=2*dimy-1
    printQQMatrx(dimx,dimy,List(
      calculateQQMatrix(dimx,dimy,nx,ny,getMomensFromConsant_Sample(nx,ny,w,h))
    ))
  }

  abstract class InterpolatedImage(val dimx:Int,val dimy:Int,val w:Int,val h:Int,val r:Array[Double]) {
    val nx=2*dimx-1
    val ny=2*dimy-1
    val dim=dimx*dimy
    val denomM=Linsystems.getInvertedMatrix(dim,dim,
      calculateQQMatrix(dimx,dimy,nx,ny,getMomensFromConsant_Sample(nx,ny,w,h)))   

    def getF(A:Array[Double]):Double

    def getImageInterpolated():java.awt.image.BufferedImage=
    {
      System.err.println("getImageInterpolated calculate enter");
      val rimg = new java.awt.image.BufferedImage(w, h, java.awt.image.BufferedImage.TYPE_BYTE_GRAY /*TYPE_USHORT_GRAY*/)
      val scalex:Double=w-1
      val scaley:Double=h-1
      val wr=rimg.getRaster()
      val px=new Array[Double](1)     
      val A=new Array[Double](dim)
      val bX=new Array[Array[Double]](w)
      for (ox <- 0 until w) bX(ox)=Q.getX0Moments(dimx,2*ox/scalex-1)

      for (oy <- 0 until h) {
        val Y=Q.getX0Moments(dimy,2*oy/scaley-1)
        for (ox <- 0 until w) {
          val X=bX(ox)         
          for(my <- 0 until dimy ; mx <- 0 until dimx) A(mx+dimx*my)=X(mx)*Y(my)
          px(0)=getF(A)
          wr.setPixel(ox, oy, px)
        }
      }

      var smsre=0.0
      var n2msre=0.0
      for (oy <- 0 until h ; ox <- 0 until w ){
        wr.getPixel(ox, oy, px)
        smsre+=java.lang.Math.pow(px(0)-origraster(ox+oy*w),2)
        n2msre+=java.lang.Math.pow(origraster(ox+oy*w),2)
      }
      System.err.println("getImageInterpolated calculate exit msre="+(smsre/n2msre));
    rimg
    }
  }

  def getRadonNikodymInterpolated(dimx:Int,dimy:Int)={
    new InterpolatedImage(dimx,dimy,image.getWidth(),image.getHeight(),origraster){
       val nomM=TMatr.getMatr_x_Matr(dim,denomM,dim,TMatr.getMatr_x_Matr(dim,
         calculateQQMatrix(dimx,dimy,nx,ny,getMomentFromRaster(nx,ny,w,h,r)),dim,denomM,dim),dim)
      def getF(A:Array[Double])=TMatr.getAMatrA(A,nomM,dim)/TMatr.getAMatrA(A,denomM,dim)
    }.getImageInterpolated()
  }

  def getLeastSquaresInterpolated(dimx:Int,dimy:Int)={
    new InterpolatedImage(dimx,dimy,image.getWidth(),image.getHeight(),origraster){
      val Gfmoments=TMatr.getMatr_x_A(dim,denomM,dim,getMomentFromRaster(dimx,dimy,w,h,r))
      def getF(A:Array[Double])=TMatr.mulVV(dim,Gfmoments,A)
    }.getImageInterpolated()
  }

}

object ExampleImageInterpolation {
  def main(args: Array[String]) {
    if(args.length!=4)
    {
      System.err.println("usage: ExampleImageInterpolation file.png dimX dimY {legendre|chebyshev}")
      System.exit(1)
    }
    val dimX=java.lang.Integer.valueOf(args(1))
    val dimY=java.lang.Integer.valueOf(args(2))
    val Q=if(args(3).equals("legendre")) new OrthogonalPolynomialsLegendreBasis()
      else if(args(3).equals("chebyshev")) new OrthogonalPolynomialsChebyshevBasis()
      else throw new RuntimeException("Unsupported basis "+args(3))

    val E=new ExampleImageInterpolation(args(0),Q)    
    E.saveImage(".same",E.getImageFromRaster(E.image.getWidth(),E.image.getHeight(),E.origraster))
    E.saveImage(".LS."+dimX+"."+dimY+".sampleG",E.getLeastSquaresInterpolated(dimX,dimY))
    E.saveImage(".RN."+dimX+"."+dimY+".sampleG",E.getRadonNikodymInterpolated(dimX,dimY))


    //E.printConstantMoments(5,5,500,150)
    //E.printQQConstant(3,3,500,150)
  }
}
