package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils._

/** A simple test (x1,x2,x3,...)_l -> y_l.
  * @version $Id: ExampleInterpolationVectorML.scala,v 1.4 2015/12/12 09:45:10 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  * A library from http://www.ioffe.ru/LNEPS/malyshkin/code_polynomials_quadratures.zip is required.
  */
class ExampleInterpolationVectorML(val Q:OrthogonalPolynomialsABasis) {

  val RANDOM=new java.util.Random(123)
  val Runge=new PolynomialsRatio(Array(1,0,25),Array(1),new OrthogonalPolynomialsMonomialsBasis())
  val FUNCS=Array((x:Double)=>x,(x:Double)=>Runge.getF(x),(x:Double)=>if(x<=0) 0.0 else 1.0)

  def getSampleInterpolatory(dimx:Int,M:Int,R:Double,f:(Double)=>Double)={
    val r=new Array[OneYObservationPoint](M)
    for(l <- 0 until M)
    {
      val x0=2*RANDOM.nextDouble()-1
      val y=f(x0)
      val x=x0+R*(2*RANDOM.nextDouble()-1)
      r(l)=new OneYObservationPoint(y,getXVector(dimx,x))
    }
    r
  }
  def getXVector(dimx:Int,x:Double)=Q.getX0Moments(dimx,x)

def printInterpolatoryType(txt:String,dimX:Int,M:Int,R:Double,f: (Double) => Double) = {
  val sample=getSampleInterpolatory(dimX,M,R,f)
  val yd=new VectorML(sample,Q)
  val ynodes=yd.getYk()
  val (aCoverage,aDisclassification)=yd.getCoverageAndDisclassification()
  val fprefix="V.R="+R+".d="+yd.dim+"."+txt
  val fname=fprefix+".dat";
  val xmin= -1.5
  val xmax= 1.5
  val step=(xmax-xmin)/10000
  System.err.print("Writing "+fname);
  val of=new java.io.PrintWriter(fname)
  of.print("|x\ty\tLS\tRN")
  for(k <- 0 until ynodes.length)  of.print("\tP("+ynodes(k)+")")
  of.println()
  for(x <- xmin to xmax+step/2 by step)
  {
    val xv=getXVector(yd.dim,x)
    val pP=yd.getP(xv)
    val y=f(x)
    of.print(""+x+"\t"+y+"\t"+yd.getYLSQ(xv)+"\t"+yd.getYRN(xv))
    for(k <- 0 until yd.dim) {
      for(k <- 0 until pP.length) of.print("\t"+pP(k))
    }
    of.println()
  }
  of.close()
  System.err.println(" DONE");

  val fn=fprefix+".Q.dat"
  System.err.print("Writing "+fn);
  val o=new java.io.PrintWriter(fn)
  o.println("|y\tCoverage\tDisclassification")
  for(i <- 0 until ynodes.length)
  {
    o.println(String.valueOf(ynodes(i))+"\t"+aCoverage(i)+"\t"+aDisclassification(i))
  }
  o.close()
  System.err.println(" DONE");
}
}

object ExampleInterpolationVectorML {

  def main(args: Array[String]) {
    val M=10000
    for(dimX <- List(20,10)) {
      val E=new ExampleInterpolationVectorML(new OrthogonalPolynomialsChebyshevBasis()/* new OrthogonalPolynomialsLegendreBasis()*/)
      for(R <- List(0.01,0.1,0.3,0.5))
      {
        E.printInterpolatoryType("X",dimX,M,R,E.FUNCS(0))
        E.printInterpolatoryType("Runge",dimX,M,R,E.FUNCS(1))
        E.printInterpolatoryType("Step",dimX,M,R,E.FUNCS(2))
      }
    }
  }
}
