/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */

/** Skweness estimator.
 * $Id: Skewness.scala,v 1.8 2014/05/27 14:36:31 mal Exp $
 *  @version $Id: Skewness.scala,v 1.8 2014/05/27 14:36:31 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
object Skewness {
  def getChi2ExactMoments(n: Int,n_degree :Int) : Array[Double]={    
    val m=new Array[Double](n);
    var s=1.0;
    for(k <- 0 until n) {
      m(k)=s
      s *= (n_degree+2*k)
    }
    m
  }

  def getFromMom(m: Array[Double]): Array[Double]={
    Array(m(4),-4*m(3),6*m(2),-4*m(1),m(0));
  }
 
  def getD1P(f:Array[Double],x:Double): Double ={
    var i=0.0
    for(k <- f.length-1 to 1 by -1) {
	i=i*x+k*f(k)
    }
    i
  }
  def getD2P(f:Array[Double],x:Double): Double ={
    var i=0.0
    for(k <- f.length-1 to 2 by -1) {
	i=i*x+(k*(k-1))*f(k)
    }
    i
  }


  class Quadrature(val x:Array[Double],val w:Array[Double]) {   
    assert(x.length == w.length)
  }

  def get2PQuadraturefromMoments(m: Array[Double]) :Quadrature ={
    val d= m(2)*m(0)-m(1)*m(1)
    val a= (m(3)*m(1)-m(2)*m(2))/d
    val b= (m(2)*m(1)-m(3)*m(0))/d
    //println("a="+a+"\tb="+b);
    val sq=Math.sqrt(b*b-4*a)
    val x1=0.5*(-b-sq)
    val x2=0.5*(-b+sq)
    val xa=m(1)/m(0)
    val w1=m(0)*(xa-x2)/(x1-x2)
    val w2=m(0)*(x1-xa)/(x1-x2)
    //println("p1\t"+x1+"\t"+w1)
    //println("p2\t"+x2+"\t"+w2)
    //println("A:Modified skewness="+((2*xa-x1-x2)/(x1-x2)))
    //println("B:Modified skewness="+(-(2*xa+b)/Math.sqrt(b*b-4*a)))
    new Quadrature(Array(x1,x2),Array(w1,w2))
  }

  def getRegularSkewness(m:Array[Double]):Double ={
    val F=getFromMom(m)
    val xa=m(1)/m(0)
    (-1.0/4)*(getD1P(F,xa)/m(0))/Math.pow(getD2P(F,xa)/m(0)/12,3.0/2.0)
  }
  def getMSkewness(m:Array[Double]):Double ={
    val q=get2PQuadraturefromMoments(m);
    (q.w(0)-q.w(1))/(q.w(0)+q.w(1))
  }


  def main(args: Array[String]) {
    println("#k\treg\tmod");
    for(k <- 1 to 150)
    {
      val m=getChi2ExactMoments(5,k)
      println(String.valueOf(k)+"\t"+getRegularSkewness(m)+"\t"+getMSkewness(m)
        +"\t"+com.polytechnik.utils.Skewness.getSkewnessFromMonomialMoments(m(0),m(1),m(2),m(3)).sk
      )      
    }
  }
    
}
