package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils._

/** A vector  (x1,x2,x3,...)_l -> y_l ML.
  * @version $Id: VectorML.scala,v 1.6 2015/12/12 09:45:10 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
class VectorML(val sample:Array[OneYObservationPoint],val Q:OrthogonalPolynomialsABasis)
{
  val dim=sample(0).x.length

  val yQQ=new Array[Double](dim*dim)
  val QQ=new Array[Double](dim*dim)
  val momY=new Array[Double](dim)

; for(j <- 0 until dim)
  {
    var sym=0.0;
    for(l <- 0 until sample.length)
    {
      assert(dim == sample(l).x.length)
      sym+=sample(l).x(j)*sample(l).y
    }
    momY(j)=sym

    for(k <- 0 until dim )
    {
      var s=0.0;
      var sy=0.0;
      for(l <- 0 until sample.length)
      {
        val v=sample(l).x(j)*sample(l).x(k)
        s+=v
        sy+=v*sample(l).y
      }
      yQQ(j*dim+k)=sy
      QQ(j*dim+k)=s
    }    
  }

  val ev=Q.EV.getGEV_Generalized(dim,yQQ,dim,QQ,dim,true);
  // a better option is to use ev, that is already diagonal
  // we invert the QQ matrix only for demonstration of the algoritm
  val IQQ=Linsystems.getInvertedMatrix(dim,dim,QQ)

 
  def getCoverageAndDisclassification()={
    val C=new Array[Double](dim)
    val D=new Array[Double](dim)
    val w=new Array[Double](dim)
    for(l <- 0 until sample.length)
    {
      val dl=TMatr.getAMatrA(sample(l).x,IQQ,dim)
      for( i <- 0 until dim) w(i)=Math.pow(TMatr.mulVV(dim,ev,dim+i*dim,sample(l).x,0),2)/dl
      for( i <- 0 until dim)
      {
        C(i)+=w(i)
        for(j <- 0 until dim ; if j !=i) D(i)+=w(i)*w(j)
        //System.err.println("w.sum="+w.sum+" must be =1")
      }
    }
    (C,D)
  }
  


  val (wc,dc)=getCoverageAndDisclassification()
  for(i <- 0 until wc.length) System.err.println("i="+i+" coverage="+wc(i)+" disclassification="+dc(i))
  System.err.println("sum(coverage)="+wc.sum+" must be ="+sample.length+" sum(disclassification)="+dc.sum)


  def getYLSQ(x:Array[Double])={
    assert(dim == x.length)
    TMatr.getAMatrB(x,momY,IQQ,dim)
  }
  def getYRN(x:Array[Double])={
    assert(dim == x.length)
    TMatr.getAMatrA(TMatr.getMatr_x_A(dim,IQQ,dim,x),yQQ,dim)/
    TMatr.getAMatrA(x,IQQ,dim)
  }

  def getYk()=ev.slice(0,dim)

  def getWno2(x:Array[Double])={
    assert(dim == x.length)
    val w=new Array[Double](dim)
    for(k <- 0 until dim)
    {
      w(k)=TMatr.mulVV(dim,x,0,ev,dim+k*dim)
    }
    w
  }
 
  def getW(x:Array[Double])=getWno2(x).map( Math.pow(_,2))

  def getP(x:Array[Double])={
    val w=getW(x)
    w.map(_ / w.sum)
  }

}

